<?php

namespace modules\catalogs\services;

use modules\catalogs\Models\Category;
use modules\catalogs\Models\ProductCustomerPrice;
use modules\catalogs\Models\Products;
use modules\catalogs\Models\ProductStock;
use modules\catalogs\Models\ProductSubLocation;
use modules\catalogs\Models\ProductSupplierPrice;

class CategoryService
{
    protected $all = [
        [
            'id' => -1,
            'title' => 'Tous',
            'parent_id' => null,
            'backcolor' => "#ffffff",
            'color' => "#000000",
            "slug" => "all",
            'children' => []
        ]
    ];

     /**
     * Récupère les catégories en fonction des filtres (parent, category).
     * 
     * @param array $filters
     * @return array
     */
    public function categories(array $filters = []): array
    {
        $isChild = false;
        $categories = [];
        $parent = null;

        if (isset($filters['parent'])) {
            // Récupérer les sous-catégories d'un parent donné
            $categories = Category::select('id', 'title', 'image', 'background', 'color', 'slug', 'parent_id')
                ->where('parent_id', $filters['parent'])
                ->where('status', 'active')
                ->get();

            $parent = Category::find($filters['parent']);
            $isChild = true;
        } elseif (isset($filters['category'])) {
            // Récupérer les sous-catégories via le slug de la catégorie
            $parent = Category::where('slug', $filters['category'])->where('status', 'active')->first();            
            if ($parent) {
                $categories = Category::select('id', 'title', 'image', 'background', 'color', 'slug', 'parent_id')
                    ->where('parent_id', $parent->id)
                    ->where('status', 'active')
                    ->get();

                #dd($parent, $categories);
                $isChild = true;

                // Si aucune sous-catégorie, retourner les catégories racines
                if ($categories->isEmpty()) {
                    $categories = Category::select('id', 'title', 'image', 'background', 'color', 'slug')
                        ->where('parent_id', $parent->parent_id)
                        ->where('status', 'active')
                        ->get();

                    $parent = Category::find($parent->parent_id);
                    $categories = array_merge($this->all, $categories->toArray());     
                    $isChild = true;
                }
            }
        } else {
            // Récupérer les catégories racines
            $categories = Category::select('id', 'title', 'image', 'background', 'color', 'slug')
                            ->whereNull('parent_id')
                            ->where('status', 'active')
                            ->get();
            $categories = array_merge($this->all, $categories->toArray());
        }

        return [
            'categories' => $categories,
            'isChild' => $isChild,
            'parent' => $parent,
        ];
    }
}