<?php

namespace modules\sales\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Session;
use modules\crm\Models\BusinessContact;
use modules\crm\services\CrmService;
use modules\location\Models\Location;
use modules\sales\Models\Caisse;
use modules\sales\Models\CaisseStatus;
use modules\sales\services\BasketService;
use modules\sales\services\CaisseService;

class salesController extends Controller
{
    protected $basket;
    public function __construct(BasketService $basket)
    {
        $this->basket = $basket;
    }

    public function index()
    {
        #dd(documenttype());
        #dd(Session::has('caisse_id'));
        if(Session::has('caisse_id')){
            $caisse_id = Session::get("caisse_id");
            $caisse = CaisseService::status($caisse_id);

            if($caisse['status'] == 'close'){
                return Redirect::route('app.receipt.caisse.open');
            }

            if(Session::has("vente_token")){
                $token = Session::get("vente_token");

                $locations = Location::where('type', 'site')->select('id', 'type', 'title')->get();
                $location = my_location();

                $customers = CrmService::customers();        
                return view('sales::receipt.index', [
                    'locations' => [
                        'sites' => $locations,
                        'selected' => $location
                    ],
                    'caisse' => [
                        'data' => Caisse::find($caisse_id),
                        'amount' => $caisse["amount"]
                    ],
                    'customers' => $customers->toArray()
                ]);
            }else{
                return Redirect::route('app.receipt.caisse.open');                
            }

            
        }else{
            return Redirect::route('app.receipt.caisse.open');
        }
    }    
    
    public function openCaisse(Request $request)
    {
        if($request->isMethod('POST')){
            #dd($request->all());            
            $caisse = CaisseService::open($request);
            $caisse_name = $caisse['caisse']['data']['title'];

            $token = generateString(60);
            Session::put("vente_token", $token);

            return response()->json([
                'status' => 'success',
                'caisse' => $caisse,
                'url' => [
                    'sale' => route('app.receipt.index')
                ],
                'message' => "Caisse [{$caisse_name}] ouvert succès"
            ]);
        }

        $locations = Location::where('type', 'site')->select('id', 'type', 'title')->get();
        $location = my_location();      
        $caisses = Caisse::where('status', 'active')->get();
        return view('sales::receipt.open', [
            'locations' => [
                'sites' => $locations,
                'selected' => $location
            ],
            'caisses' => $caisses
        ]);
    }  
    
    public function caisseStatus(Request $request)
    {        
        $caisse_id = $request->input('caisse');        
        $location_id = $request->input('location');        

        $status = CaisseService::status($caisse_id);
        $caisse = CaisseStatus::where('location_id', $location_id)->where('caisse_id')->orderByDesc('created_at')->first();
        
        return response()->json([
            'status' => 'success',
            'caisse' => [
                'status' => $status,
                'data' => $caisse
            ]
        ]);
    } 
        
}