<?php

namespace modules\stocks\Http\Controllers;

use App\Http\Controllers\Controller;
use modules\stocks\Models\Appro;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use modules\catalogs\Models\Category;
use modules\catalogs\Models\Container;
use modules\catalogs\Models\ProductLocation;
use modules\catalogs\Models\Products;
use modules\catalogs\Models\ProductStock;
use modules\catalogs\Models\ProductSupplierPrice;
use modules\catalogs\services\CategoryService;
use modules\catalogs\services\ProductService;
use modules\catalogs\services\StockService;
use modules\crm\Models\BusinessContact;
use modules\crm\services\CrmService;
use modules\location\Models\Location;
use modules\stocks\Models\ApproDetail;
use modules\stocks\Models\StockControl;
use modules\stocks\Models\StockStory;

class approController extends Controller
{

    protected $stock;
    protected $product;
    protected $category;

    public function __construct(StockService $stock, ProductService $product, CategoryService $category)
    {   
        $this->stock = $stock;
        $this->product = $product;
        $this->category = $category;
    }

    protected $all = [
        [
            'id' => -1,
            'title' => 'Tous',
            'parent_id' => null,
            'backcolor' => "#ffffff",
            'color' => "#000000",
            "slug" => "all",
            'children' => []
        ],
        [
            'id' => 0,
            'title' => 'Favoris',                
            'parent_id' => null,
            'backcolor' => "#ffffff",
            'color' => "#000000",
            "slug" => "favorites",
            'children' => []
        ],
    ];

    public function index()
    {
        return view('stocks::appros.index');
    }

    public function appros(Request $request){
        $page = isset($request->pagesize) ? $request->pagesize : 25;
        $location = Location::find(my_location()->id);
        $res = Appro::with('details');
        if((int)$location->is_parent == 0){
            $res->where('location_id', $location->id);
        }        

        if($request->has('slug')){
            $res->where('token', $request->input('slug'));
            $page = 0;
        }

        if ($request->has('criteria')) {
            if ($request->criteria == 'title') {
                $res->where('appros.title', 'LIKE', "%" . $request->keyword . "%");
            }
            if ($request->criteria == 'date_appro') {
                $res->whereDate('appros.date_appro', $request->keyword);
            }
            if ($request->criteria == 'location') {
                $res->where('appros.location_id', $request->keyword['value']);
            }
            if ($request->criteria == 'status') {
                if($request->keyword['value'] == 'cancelled'){                    
                    $res->whereNotNull('appros.cancelled_at');                    
                }
                if($request->keyword['value'] == 'validate'){                    
                    $res->whereNull('appros.cancelled_at');                    
                }
            }
        }

        $res->orderBy('date_appro');
        if((int)$page == 0){
            $appros = $res->get();
        }else{
            $appros = $res->paginate($page);
        }

        $permissions = array_values(auth()->user()->getAllPermissions()->filter(function ($permission) {
            return str_contains($permission->name, 'appro');
        })->toArray());        

        $suppliers = CrmService::suppliers()->select('id', 'title');
        $containers = Container::select('id', 'title')->get()->toArray();
        $containers[] = [
            'id' => 0,
            'title' => 'Autre'
        ];

        if(is_null(Auth::user()->location_id)){
            $locations = Location::where('status', 'active')->where('stock_management', 1)->get();
        }else{
            $locations = Location::where('id', Auth::user()->location_id)->where('stock_management', 1)->get();
        }
        
        return response()->json([
            'status' => 'success',
            'appros' => $appros,
            'permissions' => $permissions,
            'suppliers' => $suppliers,
            'containers' => $containers,
            'locations' => $locations
        ]);
    }
    
    public function create(Request $request)
    {
        if($request->ajax()){
            $locations = Location::where('type', 'site')->where('stock_management', 1)->select('id', 'type', 'title')->get();        
            $location = my_location();

            $suppliers = CrmService::suppliers()->select('id', 'title');
            $containers = Container::select('id', 'title')->get()->toArray();
            $containers[] = [
                'id' => 0,
                'title' => 'Autre'
            ];

            return response()->json([
                'suppliers' => $suppliers,
                'containers' => $containers,
                'settings' => settings(),
                'locations' => [
                    'sites' => $locations,
                    'selected' => $location,
                    'edit' => Auth::user()->location_id == null
                ]
            ]);
        }

        return view('stocks::appros.create');
    }

    public function detail(string $slug)
    {
        $locations = Location::where('type', 'site')->select('id', 'type', 'title')->get();
        $location = my_location();

        $suppliers = CrmService::suppliers()->select('id', 'title');
        $containers = Container::select('id', 'title')->get()->toArray();

        return view('stocks::appros.detail', [
            'suppliers' => $suppliers,
            'containers' => $containers,
            'locations' => [
                'sites' => $locations,
                'selected' => $location
            ]
        ]);
    }    

    public function store(Request $request){  
        try {
            $request->validate([
                'title' => "required",
                'details' => "required|array|min:1",
                'supplier.id' => 'required',
                'container.selected.id' => 'required',
            ]);            

            #dd($request->all());

            DB::beginTransaction();

            $location_id = $request->location['id'];            
            $user = Auth::user();

            $container = $request->container['selected'];            
            if((int)$container['id'] == 0){
                $container = Container::create([
                    'title' => $request->container['other'],
                    'created_at' => date_now(),
                    'updated_at' => date_now(),
                ])->toArray();
            }

            $supplier = $request->supplier;
            $appro = Appro::create([
                'location_id' => $request->location['id'],
                'location' => $request->location['title'],
                'container_id' => $container['id'],
                'container' => $container['title'],
                'supplier_id' => $supplier['id'],
                'supplier' => $supplier['title'],
                'title' => $request->title,
                'document' => $request->document,
                'date_appro' => $request->date,
                'observation' => $request->observation,
                'user_id' => $user->id,
                'username' => $user->name,
                'created_at' => date_now(),
                'updated_at' => date_now(),
            ]);

            $id = $appro->id;
            $str = generateString(12);
            $token = "{$id}-{$str}";
            $appro->token = $token;
            $appro->save();

            $details = $request->details;
            foreach ($details as $key => $item) {
                $product_id = $item['id'];            
                $qty_appro = $item['qty']['appro'];                
                
                $where = [
                    'product_id' => $product_id,
                    'location_id' => $location_id
                ];                
                
                $stock_actual = $item['qty']['initial'];
                $stock_final = (double)$item['qty']['initial'] + (double)$item['qty']['appro'];

                $price = $item['price'];
                #dd($price);
                ApproDetail::create([
                    "appro_id" => $id,
                    "product_id" => $product_id,
                    'product_reference' => $item['reference'],
                    'product_title' => $item['title'],
                    'frns_price' => $price,
                    'stock_in' => $qty_appro,
                    'stock_actual' => $stock_actual,
                    'stock_final' => $stock_final,
                    'unite_id' => $item['unite']['id'],
                    'unite' => $item['unite']['title'],
                    'row_index' => $key,
                    'token' => $token,
                    'created_at' => date_now(),
                    'updated_at' => date_now(),
                ]);

                $exist = ProductSupplierPrice::where([
                            'product_id' => $product_id,
                            'supplier_id' => $supplier['id'],
                            'location_id' => $location_id
                        ])->latest()->first();

                ProductLocation::where([
                    'product_id' => $product_id,                    
                    'location_id' => $location_id
                ])->update([
                    'frns_id' => $supplier['id'],
                    'frns' => $supplier['title'],
                ]);

                // Vérifier si l'enregistrement existe et si les valeurs sont différentes
                if (!$exist || $exist->price != $price) {
                    // Insérer un nouvel enregistrement si les valeurs sont différentes
                    
                    ProductSupplierPrice::create([
                        'product_id' => $product_id,
                        'supplier_id' => $supplier['id'],
                        'location_id' => $location_id,
                        'price' => $price,
                        'created_at' => date_now(),
                        'updated_at' => date_now(),
                    ]);
                }

                $this->stock->update([
                    'product_id' => $product_id, 
                    'location_id' => $location_id, 
                    'quantity' => $qty_appro, 
                    'price' => $price
                ], 'entry');

                $this->stock->createUpdateControl([
                    'type' => 'in',
                    'where' => $where,
                    'location_id' => $location_id,
                    'product_id' => $product_id,
                    'product_name' => $item['title'],
                    'stock_actual' => $stock_actual,
                    'quantity' => $qty_appro,
                    'stock_final' => $stock_final,
                    'title' => "APPRO: " . $request->title,
                ]);

                $this->stock->logStockStory([
                    'product_id' => $product_id,
                    'product_name' => $item['title'],
                    'location_id' => $location_id,                    
                    'title' => "APPRO: ". $request->title,   
                    'stock_actual' => $stock_actual,                 
                    'quantity' => $qty_appro,
                    'stock_final' => $stock_final,
                    'token' => $token,
                ]);
            }

            
            DB::commit();

            $categories = $this->category->categories();
            $products = ProductService::products(['location' => $location_id]);            
            $location = Location::where('id', $location_id)->select('id', 'type', 'title')->first();
            
            return response()->json([
                "status" => "success",
                "message" => "Approvisionnement enregistrer et valider avec succès",
                'categories' => [
                    'data' => $categories['categories'],
                    'parent' => $categories['parent'],
                    'child' => false,
                ],
                'products' => $products,
                'location' => $location

            ]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }              
    }

    public function destroy(Request $request)
    {
        #try {
            #DB::beginTransaction();

            // Supposons que vous voulez supprimer une ressource par son ID
            $id = $request->input('id');

            // Effectuer la logique de suppression, par exemple, supprimer un enregistrement d'une base de données
            $item = Appro::find($id);
             
            if ($item) {
                $location_id = $item->location_id;
                Appro::where('id', $id)->update([
                    'is_cancelled' => 1,
                    'cancelled_at' => date_now(),
                    'cancelled_by' =>  json_encode([
                        'id' => Auth::user()->id,
                        'name' => Auth::user()->name,
                    ]),
                ]);

                $details = ApproDetail::where('appro_id', $id)->get();
                
                foreach ($details as $key => $item) {  
                    $product_id = $item->product_id;   
                    $product = ProductService::products(['id' => $product_id, 'location' => $location_id]);
                    $product = $product[0]->toArray();                    

                    $qty_appro = $item->stock_in;                
                    
                    $where = [
                        'product_id' => $product_id,
                        'location_id' => $location_id
                    ];                    
                    
                    $this->stock->update([
                        'product_id' => $product_id, 
                        'location_id' => $location_id, 
                        'quantity' => $qty_appro,
                        'price' => $item['frns_price']
                    ], 'back');

                    $this->stock->createUpdateControl([
                        'type' => 'out',
                        'where' => $where,
                        'location_id' => $location_id,
                        'product_id' => $product_id,
                        'quantity' => $qty_appro * -1,
                    ]);

                    $stock = $product['stock'][0];
                    $this->stock->logStockStory([
                        'product_id' => $product_id,
                        'product_name' => $item['product_title'],
                        'location_id' => $location_id,                    
                        'title' => "APPRO (ANNULATION): ". $request->title,   
                        'stock_actual' => $stock['stock'],
                        'quantity' => $qty_appro,
                        'stock_final' => (float)$stock['stock'] - (float)$qty_appro,
                        'token' => $item->token,
                    ]);
                }

                return response()->json([
                    'status' => 'success',
                    'message' => 'Appro deleted successfully.'
                ]);
            } else {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Appro not found.'
                ], 404);
            }

            #DB::commit();
        /*} catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }*/
    }
}